@@ -3,7 +3,7 @@ use qttypes::QString;
 use std::ffi::c_void;
 
 #[cfg(feature = "qmetaobject")]
-use qmetaobject::{QObject, QObjectPinned, QmlEngine};
+use qmetaobject::QmlEngine;
 
 cpp! {{
     #include <KLocalizedContext>
@@ -12,29 +12,18 @@ cpp! {{
     #include <QtQuick/QtQuick>
 
     struct KLocalizedContextHolder {
-        std::shared_ptr<KLocalizedContext> klocalized;
+        std::unique_ptr<KLocalizedContext> klocalized;
 
-        KLocalizedContextHolder() : klocalized(new KLocalizedContext) {}
         KLocalizedContextHolder(QObject *parent) : klocalized(new KLocalizedContext(parent)) {}
     };
 }}
 
 cpp_class!(
     /// Struct representing [KLocalizedContext](https://api.kde.org/frameworks/ki18n/html/classKLocalizedContext.html). Mainly used with QML.
-    #[derive(Default)]
     pub unsafe struct KLocalizedContext as "KLocalizedContextHolder"
 );
 
 impl KLocalizedContext {
-    #[cfg(feature = "qmetaobject")]
-    /// Construct  a new KLocalizedContext from a QObject.
-    pub fn new<T: QObject + Sized>(obj: QObjectPinned<T>) -> Self {
-        let obj_ptr = obj.get_or_create_cpp_object();
-        cpp!(unsafe [obj_ptr as "QObject *"] -> KLocalizedContext as "KLocalizedContextHolder" {
-            return KLocalizedContextHolder(obj_ptr);
-        })
-    }
-
     #[cfg(feature = "qmetaobject")]
     /// Initialize KLocalizedContext from Engine.
     /// **Feature** `qmetaobject` needs to be enabled for this function.
@@ -46,10 +35,12 @@ impl KLocalizedContext {
     /// let engine = QmlEngine::new();
     /// KLocalizedContext::init_from_engine(&engine);
     /// ```
-    pub fn init_from_engine(engine: &QmlEngine) {
+    pub fn init_from_engine(engine: &QmlEngine) -> Self {
         let engine_ptr = engine.cpp_ptr();
-        cpp!(unsafe [engine_ptr as "QQmlEngine*"]{
-            engine_ptr->rootContext()->setContextObject(new KLocalizedContext(engine_ptr));
+        cpp!(unsafe [engine_ptr as "QQmlEngine*"] -> KLocalizedContext as "KLocalizedContextHolder" {
+            auto klocalizedholder = KLocalizedContextHolder(engine_ptr);
+            engine_ptr->rootContext()->setContextObject(klocalizedholder.klocalized.get());
+            return klocalizedholder;
         })
     }
 
@@ -71,7 +62,7 @@ use ki18n::klocalizedcontext::KLocalizedContext;
 use qmetaobject::QmlEngine;
 
 let engine = QmlEngine::new();
-let mut context = KLocalizedContext::default();
+let mut context = KLocalizedContext::init_from_engine(&engine);
 context.set_translation_domain("Test Domain".into());
 ```
 "##
